@@ -9,7 +9,7 @@ use Scalar::Util qw( blessed );
 use Storable qw( dclone );
 use Carp qw( croak );
 
-our $VERSION = '0.05000';
+our $VERSION = '0.06000';
 $VERSION = eval $VERSION;
 
 # sub _compatible_config() is only required as long as we support deprecated
@@ -108,7 +108,7 @@ sub options_from_model {
     }
     else {
         my $has_column = $source->has_column($label_col);
-        
+
         @defaults = map {
             [
                 $_->get_column($id_col),
@@ -350,7 +350,7 @@ sub _fill_nested {
                         @{ $rep->get_fields };
 
                     if ( defined $del_field ) {
-                        $rep->remove_element($del_field);
+                        $del_field->parent->remove_element($del_field);
                     }
                 }
             }
@@ -500,11 +500,11 @@ sub _save_relationships {
 
             my @fpkey = $dbic->related_resultset($rel)->result_source->primary_columns;
 
-            my @cond = (%{$info->{cond}}); 
-             
+            my @cond = (%{$info->{cond}});
+
             # make sure $rel is a has_one or might_have rel
             # stolen from SQL/Translator/Parser/DBIx/Class
-                
+
             my $fk_constraint;
 
             # Get the key information, mapping off the foreign/self markers
@@ -655,46 +655,46 @@ sub _insert_has_many {
 
 sub _can_insert_new_row {
     my ( $dbic, $form, $config, $repetition, $rel, $pk_field ) = @_;
-
+    
     if ( $config->{new_empty_row} ) {
         # old, deprecated behaviour
-
+        
         my $rows = $config->{new_empty_row};
-
+    
         $rows = [$rows] if ref $rows ne 'ARRAY';
-
+    
         for my $name (@$rows) {
             my ($field)
                 = grep { $_->original_name eq $name } @{ $repetition->get_fields };
-
+    
             return if !defined $field;
-
+    
             my $nested_name = $field->nested_name;
             return if !$form->valid($nested_name);
-
+    
             my $value = $form->param_value($nested_name);
             return if !length $value;
         }
     }
     else {
         # new behaviour
-
+        
         my @rep_fields = @{ $repetition->get_fields };
-
+        
         my $pk_name = $pk_field->nested_name;
-
+        
         my @constraints = grep { $_->when->{field} eq $pk_name }
                           grep { defined $_->when }
                           map { @{ $_->get_constraints({ type => 'Required' }) } }
                             @rep_fields;
-
+        
         my @required_fields;
-
+        
         if ( @constraints ) {
             # if there are any Required constraints whose 'when' clause points to
             # the PK field - check that all these fields are filled in - as
             # the PK value is missing on new reps, so the constraint won't have run
-
+            
             return if
                 notall { defined && length }
                 map { $form->param_value( $_->nested_name ) }
@@ -704,11 +704,11 @@ sub _can_insert_new_row {
         else {
             # otherwise, just check at least 1 field that matches either a column
             # name or an accessor, is filled in
-
+            
             my $result_source = $dbic->$rel->result_source;
-
+            
             #  only create a new record if (read from bottom)...
-
+            
             return if
                 none { defined && length }
                 map { $form->param_value( $_->nested_name ) }
@@ -720,7 +720,7 @@ sub _can_insert_new_row {
                     @rep_fields;
         }
     }
-
+    
     return 1;
 }
 
@@ -739,32 +739,32 @@ sub _delete_has_many {
         unless $form->valid($nested_name)
             && $form->param_value($nested_name);
 
-    $row->delete;
+    $row->delete if ($row->in_storage);
 
     return 1;
 }
 
 sub _fix_value {
     my ( $dbic, $col, $value, $field, ) = @_;
-
+    
     my $col_info    = $dbic->column_info($col);
     my $is_nullable = $col_info->{is_nullable} || 0;
     my $data_type   = $col_info->{data_type} || '';
-
+    
     if ( defined $value ) {
         if ( (     $is_nullable
                 || $data_type =~ m/^timestamp|date|int|float|numeric/i
             )
 
             # comparing to '' does not work for inflated objects
-            && !ref $value
+            && !ref $value 
             && $value eq ''
             )
         {
             $value = undef;
         }
     }
-
+    
     if ( !defined $value
         && defined $field
         && $field->isa('HTML::FormFu::Element::Checkbox')
@@ -772,7 +772,7 @@ sub _fix_value {
     {
         $value = 0;
     }
-
+    
     return $value;
 }
 
@@ -781,19 +781,19 @@ sub _save_columns {
 
     for my $field ( @{ $base->get_fields }, ) {
         next if not is_direct_child( $base, $field );
-
+        
         my $config = _compatible_config($field);
         next if $config->{delete_if_true};
         next if $config->{read_only};
-
+        
         my $name = $field->name;
         $name = $field->original_name if $field->original_name;
-
+        
         my $accessor = $config->{accessor} || $name;
         next if not defined $accessor;
-
+        
         my $value = $form->param_value( $field->nested_name );
-
+        
         next if $config->{ignore_if_empty} && ( !defined $value || $value eq "" );
 
         my ($pk) = $dbic->result_source->primary_columns;
@@ -805,7 +805,7 @@ sub _save_columns {
             && ( !defined $value || !length $value ) )
         {
             $dbic->discard_changes if $dbic->is_changed;
-            $dbic->delete;
+            $dbic->delete if $dbic->in_storage;
             return;
         }
         if ( $dbic->result_source->has_column($accessor) ) {
@@ -825,7 +825,7 @@ sub _save_columns {
         {
             $dbic->set_column( $accessor, $value );
         }
-        elsif ( $dbic->can($accessor)
+        elsif ( $dbic->can($accessor) 
             # and $accessor is not a has_one or might_have rel where the foreign key is on the foreign table
             and !$dbic->result_source->relationship_info($accessor)) {
             $dbic->$accessor($value);
@@ -847,17 +847,17 @@ sub _save_columns {
     for my $valid ( $form->valid ) {
         next if @{ $base->get_fields( name => $valid ) };
         next if not $dbic->can($valid);
-
+        
         my $value = $form->param_value($valid);
         $dbic->$valid($value);
     }
-
+    
     return 1;
 }
 
 sub _save_multi_value_fields_many_to_many {
     my ( $base, $dbic, $form, $attrs, $rels, $cols ) = @_;
-
+    
     my @fields = grep {
         ( defined $attrs->{nested_base} && defined $_->parent->nested_name )
             ? $_->parent->nested_name eq $attrs->{nested_base}
@@ -883,12 +883,13 @@ sub _save_multi_value_fields_many_to_many {
             my @values = $form->param_list($nested_name);
             my @rows;
 
+            my $config = _compatible_config($field);
+
+            my ($pk) = $config->{default_column}
+                || $related->result_source->primary_columns;
+
             if (@values) {
-                my $config = _compatible_config($field);
-
-                my ($pk) = $config->{default_column}
-                    || $related->result_source->primary_columns;
-
+                
                 $pk = "me.$pk" unless $pk =~ /\./;
 
                 @rows = $related->result_source->resultset->search( {
@@ -896,9 +897,24 @@ sub _save_multi_value_fields_many_to_many {
                         $pk => { -in => \@values } } )->all;
             }
 
-            my $set_method = "set_$name";
-
-            $dbic->$set_method( \@rows );
+            if($config->{additive}) {
+                
+                my $relinfo = $dbic->result_source->relationship_info($name);
+                
+                $pk =~ s/^.*\.//;
+                
+                my $set_method = "add_to_$name";
+                my $remove_method = "remove_from_$name";
+                                
+                foreach my $row ( @rows ) {
+                    $dbic->$remove_method($row);
+                    $dbic->$set_method($row, $config->{link_values});
+                }
+            } else {
+                my $set_method = "set_$name";
+                
+                $dbic->$set_method( \@rows, $config->{link_values} );
+            }
         }
     }
 }
@@ -1050,26 +1066,26 @@ Example of typical use in a Catalyst controller:
 
     sub edit : Chained {
         my ( $self, $c ) = @_;
-
+        
         my $form = $c->stash->{form};
         my $book = $c->stash->{book};
-
+        
         if ( $form->submitted_and_valid ) {
-
+            
             # update dbic row with submitted values from form
-
+            
             $form->model->update( $book );
-
+            
             $c->response->redirect( $c->uri_for('view', $book->id) );
             return;
         }
         elsif ( !$form->submitted ) {
-
+            
             # use dbic row to set form's default values
-
+            
             $form->model->default_values( $book );
         }
-
+        
         return;
     }
 
@@ -1104,7 +1120,7 @@ Arguments: $dbic_row, [\%config]
 
 Return Value: $form
 
-    $form->model->default_values( $row );
+    $form->model->default_values( $dbic_row );
 
 Set a form's default values from the database, to allow a user to edit them.
 
@@ -1114,7 +1130,7 @@ Arguments: [$dbic_row], [\%config]
 
 Return Value: $dbic_row
 
-    $form->model->update( $row );
+    $form->model->update( $dbic_row );
 
 Update the database with the submitted form values.
 
@@ -1124,7 +1140,7 @@ Arguments: [\%config]
 
 Return Value: $dbic_row
 
-    $form->model->create( $row );
+    my $dbic_row = $form->model->create( {resultset => 'Book'} );
 
 Like L</update>, but doesn't require a C<$dbic_row> argument.
 
@@ -1139,7 +1155,7 @@ An example of setting the ResultSet name on a Form:
     ---
     model_config:
       resultset: FooTable
-
+    
     elements:
       # [snip]
 
@@ -1182,20 +1198,20 @@ For the following DBIx::Class schema:
 
     package MySchema::Book;
     use base 'DBIx::Class';
-
+    
     __PACKAGE__->load_components(qw/ Core /);
-
+    
     __PACKAGE__->table("book");
-
+    
     __PACKAGE__->add_columns(
         id     => { data_type => "INTEGER" },
         title  => { data_type => "TEXT" },
         author => { data_type => "TEXT" },
         blurb  => { data_type => "TEXT" },
     );
-
+    
     __PACKAGE__->set_primary_key("id");
-
+    
     1;
 
 A suitable form for this might be:
@@ -1203,18 +1219,18 @@ A suitable form for this might be:
     elements:
       - type: Text
         name: title
-
+      
       - type: Text
         name: author
-
+      
       - type: Textarea
         name: blurb
 
 =head2 might_have and has_one relationships
 
-Set field values from a related row with a C<might_have> or C<has_one>
-relationship by placing the fields within a
-L<Block|HTML::FormFu::Element::Block> (or any element that inherits from
+Set field values from a related row with a C<might_have> or C<has_one> 
+relationship by placing the fields within a 
+L<Block|HTML::FormFu::Element::Block> (or any element that inherits from 
 Block, such as L<Fieldset|HTML::FormFu::Element::Fieldset>) with its
 L<HTML::FormFu/nested_name> set to the relationship name.
 
@@ -1222,40 +1238,40 @@ For the following DBIx::Class schemas:
 
     package MySchema::Book;
     use base 'DBIx::Class';
-
+    
     __PACKAGE__->load_components(qw/ Core /);
-
+    
     __PACKAGE__->table("book");
-
+    
     __PACKAGE__->add_columns(
         id    => { data_type => "INTEGER" },
         title => { data_type => "TEXT" },
     );
-
+    
     __PACKAGE__->set_primary_key("id");
-
+    
     __PACKAGE__->might_have( review => 'MySchema::Review', 'book' );
-
+    
     1;
 
 
     package MySchema::Review;
     use base 'DBIx::Class';
-
+    
     __PACKAGE__->load_components(qw/ Core /);
-
+    
     __PACKAGE__->table("review");
-
+    
     __PACKAGE__->add_columns(
         id          => { data_type => "INTEGER" },
         book        => { data_type => "INTEGER", is_nullable => 1 },
         review_text => { data_type => "TEXT" },
     );
-
+    
     __PACKAGE__->set_primary_key("book");
-
+    
     __PACKAGE__->belongs_to( book => 'MySchema::Book' );
-
+    
     1;
 
 A suitable form for this would be:
@@ -1263,7 +1279,7 @@ A suitable form for this would be:
     elements:
       - type: Text
         name: title
-
+      
       - type: Block
         nested_name: review
         elements:
@@ -1275,20 +1291,20 @@ to have a field for the related table's primary key, as DBIx::Class will
 handle retrieving the correct row automatically.
 
 You can also set a C<has_one> or C<might_have> relationship using a multi value
-field like L<Select|HTML::FormFu::Element::Select>.
+field like L<Select|HTML::FormFu::Element::Select>. 
 
     elements:
       - type: Text
         name: title
-
+      
       - type: Select
         nested: review
         model_config:
           resultset: Review
 
 This will load all reviews into the select field. If you select a review from
-that list, a current relationship to a review is removed and the new one is
-added. This requires that the primary key of the C<Review> table and the
+that list, a current relationship to a review is removed and the new one is 
+added. This requires that the primary key of the C<Review> table and the 
 foreign key do not match.
 
 =head2 has_many and many_to_many relationships
@@ -1313,45 +1329,45 @@ outside of the Repeatable block.
 This field is used to store a count of the number of repetitions of the
 Repeatable block were created.
 When the form is submitted, this value is used during C<< $form->process >>
-to ensure the form is rebuild with the correct number of repetitions.
+to ensure the form is rebuilt with the correct number of repetitions.
 
 For the following DBIx::Class schemas:
 
     package MySchema::Book;
     use base 'DBIx::Class';
-
+    
     __PACKAGE__->load_components(qw/ Core /);
-
+    
     __PACKAGE__->table("book");
-
+    
     __PACKAGE__->add_columns(
         id    => { data_type => "INTEGER" },
         title => { data_type => "TEXT" },
     );
-
+    
     __PACKAGE__->set_primary_key("id");
-
+    
     __PACKAGE__->has_many( review => 'MySchema::Review', 'book' );
-
+    
     1;
 
 
     package MySchema::Review;
     use base 'DBIx::Class';
-
+    
     __PACKAGE__->load_components(qw/ Core /);
-
+    
     __PACKAGE__->table("review");
-
+    
     __PACKAGE__->add_columns(
         book        => { data_type => "INTEGER" },
         review_text => { data_type => "TEXT" },
     );
-
+    
     __PACKAGE__->set_primary_key("book");
-
+    
     __PACKAGE__->belongs_to( book => 'MySchema::Book' );
-
+    
     1;
 
 A suitable form for this would be:
@@ -1359,29 +1375,29 @@ A suitable form for this would be:
     elements:
       - type: Text
         name: title
-
+      
       - type: Hidden
         name: review_count
-
+      
       - type: Repeatable
         nested_name: review
         counter_name: review_count
         elements:
           - type: Hidden
             name: book
-
+          
           - type: Textarea
             name: review_text
 
 =head2 many_to_many selection
 
 To select / deselect rows from a C<many_to_many> relationship, you must use
-a multi-valued element, such as a
+a multi-valued element, such as a 
 L<Checkboxgroup|HTML::FormFu::Element::Checkboxgroup> or a
-L<Select|HTML::FormFu::Element::Select> with
+L<Select|HTML::FormFu::Element::Select> with 
 L<multiple|HTML::FormFu::Element::Select/multiple> set.
 
-The field's L<name|HTML::FormFu::Element::_Field/name> must be set to the
+The field's L<name|HTML::FormFu::Element::_Field/name> must be set to the 
 name of the C<many_to_many> relationship.
 
 =item default_column
@@ -1396,6 +1412,34 @@ primary key, set C<< $field->model_config->{default_column} >>.
           model_config:
             default_column: foo
 
+If you want to set columns on the link table you can do so if you add a 
+C<link_values> attribute to C<model_config>:
+
+    ---
+    element:
+        - type: Checkboxgroup
+          name: authors
+          model_config:
+            link_values:
+              foo: bar
+
+
+The default implementation will first remove all related objects and set the
+new ones (see L<http://search.cpan.org/perldoc?DBIx::Class::Relationship::Base#set_$rel>).
+If you want to add the selected objects to the current set of objects
+set C<additive> in the C<model_config>.
+
+    ---
+    element:
+        - type: Checkboxgroup
+          name: authors
+          model_config:
+            additive: 1
+            options_from_model: 0
+
+(<options_from_model> is set to C<0> because this L</options_from_model> will try to fetch
+all objects from the result class C<Authors> if C<model_config> is specified
+without a C<resultset> attribute.)
 
 =head1 COMMON ARGUMENTS
 
@@ -1453,39 +1497,39 @@ For the following DBIx::Class schemas:
 
     package MySchema::Book;
     use base 'DBIx::Class';
-
+    
     __PACKAGE__->load_components(qw/ Core /);
-
+    
     __PACKAGE__->table("book");
-
+    
     __PACKAGE__->add_columns(
         id    => { data_type => "INTEGER" },
         title => { data_type => "TEXT" },
     );
-
+    
     __PACKAGE__->set_primary_key("id");
-
+    
     __PACKAGE__->might_have( review => 'MySchema::Review', 'book' );
-
+    
     1;
 
 
     package MySchema::Review;
     use base 'DBIx::Class';
-
+    
     __PACKAGE__->load_components(qw/ Core /);
-
+    
     __PACKAGE__->table("review");
-
+    
     __PACKAGE__->add_columns(
         book        => { data_type => "INTEGER" },
         review_text => { data_type => "TEXT" },
     );
-
+    
     __PACKAGE__->set_primary_key("book");
-
+    
     __PACKAGE__->belongs_to( book => 'MySchema::Book' );
-
+    
     1;
 
 A suitable form for this would be:
@@ -1493,7 +1537,7 @@ A suitable form for this would be:
     elements:
       - type: Text
         name: title
-
+      
       - type: Block
         nested_name: review
         elements:
@@ -1526,20 +1570,20 @@ An example of use might be:
     elements:
       - type: Text
         name: title
-
+      
       - type: Hidden
         name: review_count
-
+      
       - type: Repeatable
         nested_name: review
         counter_name: review_count
         elements:
           - type: Hidden
             name: book
-
+          
           - type: Textarea
             name: review_text
-
+          
           - type: Checkbox
             name: delete_review
             label: 'Delete Review?'
@@ -1564,6 +1608,11 @@ block.
 
 =item new_rows_max
 
+Set to the maximum number of new rows that a Repeatable block is allowed to
+add.
+
+If not set, it will fallback to the value of C<empty_rows>.
+
 =back
 
 =head2 Config options for options_from_model
@@ -1617,13 +1666,13 @@ L<DBIx::Class::ResultSet/search>.
 
 =head2 Add extra values not in the form
 
-To update values to the database which weren't submitted to the form,
+To update values to the database which weren't submitted to the form, 
 you can first add them to the form with L<add_valid|HTML::FormFu/add_valid>.
 
     my $passwd = generate_passwd();
-
+    
     $form->add_valid( passwd => $passwd );
-
+    
     $form->model->update( $row );
 
 C<add_valid> works for fieldnames that don't exist in the form.
@@ -1634,7 +1683,7 @@ You can make a field read only. The value of such fields cannot be changed by
 the user even if they submit a value for it.
 
   $field->model_config->{read_only} = 1;
-
+  
   - Name: field
     model_config:
       read_only: 1
@@ -1655,7 +1704,7 @@ See C<empty_rows> in L</"Config options for Repeatable blocks"> instead.
 Is deprecated and provided only for backwards compatability.
 Will be removed at some point in the future.
 
-See C<empty_rows> in L</"Config options for Repeatable blocks"> instead.
+See C<new_rows_max> in L</"Config options for Repeatable blocks"> instead.
 
 =head2 Range constraint
 
@@ -1669,12 +1718,12 @@ See C<empty_rows> in L</"Config options for Repeatable blocks"> instead.
 
 =head1 CAVEATS
 
-To ensure your column's inflators and deflators are called, we have to
-get / set values using their named methods, and not with C<get_column> /
+To ensure your column's inflators and deflators are called, we have to 
+get / set values using their named methods, and not with C<get_column> / 
 C<set_column>.
 
-Because of this, beware of having column names which clash with DBIx::Class
-built-in method-names, such as C<delete>. - It will have obviously
+Because of this, beware of having column names which clash with DBIx::Class 
+built-in method-names, such as C<delete>. - It will have obviously 
 undesirable results!
 
 =head1 SUPPORT
@@ -1693,23 +1742,23 @@ L<http://lists.scsys.co.uk/pipermail/html-formfu/>
 
 =head1 BUGS
 
-Please submit bugs / feature requests to
-L<http://code.google.com/p/html-formfu/issues/list> (preferred) or
+Please submit bugs / feature requests to 
+L<http://code.google.com/p/html-formfu/issues/list> (preferred) or 
 L<http://rt.perl.org>.
 
 =head1 SUBVERSION REPOSITORY
 
-The publicly viewable subversion code repository is at
+The publicly viewable subversion code repository is at 
 L<http://html-formfu.googlecode.com/svn/trunk/HTML-FormFu-Model-DBIC>.
 
-If you wish to contribute, you'll need a GMAIL email address. Then just
+If you wish to contribute, you'll need a GMAIL email address. Then just 
 ask on the mailing list for commit access.
 
-If you wish to contribute but for some reason really don't want to sign up
-for a GMAIL account, please post patches to the mailing list (although
-you'll have to wait for someone to commit them).
+If you wish to contribute but for some reason really don't want to sign up 
+for a GMAIL account, please post patches to the mailing list (although  
+you'll have to wait for someone to commit them). 
 
-If you have commit permissions, use the HTTPS repository url:
+If you have commit permissions, use the HTTPS repository url: 
 L<https://html-formfu.googlecode.com/svn/trunk/HTML-FormFu-Model-DBIC>
 
 =head1 SEE ALSO
@@ -1735,7 +1784,7 @@ Mario Minati
 
 Copyright (C) 2007 by Carl Franks
 
-Based on the original source code of L<DBIx::Class::HTMLWidget>, copyright
+Based on the original source code of L<DBIx::Class::HTMLWidget>, copyright 
 Thomas Klausner.
 
 This library is free software; you can redistribute it and/or modify
